/*
 * Copyright (c) 2018, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
import UIKit

class BLEProvisionViewController: UIViewController
{
    // MARK: Properties
    @IBOutlet weak var circleView: UIImageView!
    @IBOutlet weak var logLabel: UILabel!
    @IBOutlet weak var logView: UITextView!
    @IBOutlet weak var completeButton: UIButton!
    @IBOutlet weak var statusLabel: UILabel!
    
    // MARK: Variables
    var referenceController: BLEProvisionInfoViewController!
    let dateFormatter = DateFormatter()
    
    // MARK: Actions
    @IBAction func completeButtonTouched(_ sender: Any)
    {
        self.navigationController?.popToRootViewController(animated: true)
    }
    
    // MARK: Functions
    
    override func viewDidLoad()
    {
        super.viewDidLoad()

        logLabel.clipsToBounds = true
        completeButton.clipsToBounds = true
        logView.clipsToBounds = true
        logView.layer.cornerRadius = 8.0
        logLabel.layer.cornerRadius = 8.0
        completeButton.layer.cornerRadius = 8.0
        dateFormatter.dateFormat = "HH:mm:ss"
        
        NotificationCenter.default.addObserver(self,
                                               selector: #selector(self.viewRotated),
                                               name: NSNotification.Name.UIDeviceOrientationDidChange, object: nil)
        
        viewRotated()
        circleView.rotate360Degrees()
        
        self.printLogMessage(message: "BLE Provisioning Started!")
        
    }
    
    override func viewWillDisappear(_ animated: Bool)
    {
        if(referenceController.provisioningComplete)
        {
            navigationController?.popToRootViewController(animated: true)
        }
    }
    
    override func viewDidAppear(_ animated: Bool)
    {
        referenceController.startProvisioning()
    }
    
    func printLogMessage(message: String)
    {
        DispatchQueue.main.async
            {
                self.logView.text = self.logView.text + self.dateFormatter.string(from: Date()) + ":    "
                                        + message + "\n"
                let range = NSRange(location: self.logView.text.characters.count-1,length: 0)
                self.logView.scrollRangeToVisible(range)
        }
        
    }
    
    @objc func viewRotated()
    {
        DispatchQueue.main.async
            {
                if UIDeviceOrientationIsPortrait(UIDevice.current.orientation)
                {
                    self.logView.isHidden = false
                    self.logLabel.isHidden = false
                }
                
                if UIDeviceOrientationIsLandscape(UIDevice.current.orientation)
                {
                    self.logView.isHidden = true
                    self.logLabel.isHidden = true
                }
        }
    }

}
