/*
 * Copyright (c) 2018, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
import UIKit

class CodeExampleTableViewController: UIViewController, UITableViewDataSource, UITableViewDelegate
{
    // MARK: Structure Definition
    struct CodeExample
    {
        var title = String()
        var image = UIImage()
    }
    
    // MARK: Code example list
    let examples =
    [
        CodeExample(title: "Project Zero",
                    image: UIImage(named: "projectzero")!),
        CodeExample(title: "Simple Application Processor",
                    image: UIImage(named: "simpleap")!),
        CodeExample(title: "Temperature Sensor Broadcast",
                    image: UIImage(named: "tempsensor")!),
        CodeExample(title: "Sensors BoosterPack",
                    image: UIImage(named: "sensorsbp")!),
        CodeExample(title: "Over the Air Download (OAD)",
                    image: UIImage(named: "oad")!)
    ]
    
    // MARK: Variables
    let cellIdentifier = "CodeExampleTableViewCell"
    var targetCodeExample = ""
    
    // MARK: Properties
    @IBOutlet weak var codeExampleTable: UITableView!
    @IBOutlet weak var dirLabel: UILabel!
    
    // MARK: Functions
    
    override func viewDidLoad()
    {
        super.viewDidLoad()
        dirLabel.adjustsFontSizeToFitWidth = true
        codeExampleTable.dataSource = self
        codeExampleTable.delegate = self
    }

    override func didReceiveMemoryWarning()
    {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    // MARK: - Table view data source
    func tableView(_ tableView: UITableView,
                   numberOfRowsInSection section: Int) -> Int
    {
        return examples.count
    }
    
    func tableView(_ tableView: UITableView,
                   cellForRowAt indexPath: IndexPath) -> UITableViewCell
    {
        let cell : CodeExampleTableViewCell =
            codeExampleTable.dequeueReusableCell(withIdentifier: cellIdentifier)! as! CodeExampleTableViewCell
        
        let codeExample = examples[indexPath.row]
        
        cell.titleText.text = codeExample.title
        cell.exampleImage.image = codeExample.image
        
        return cell
    }
    
    func tableView(_ tableView: UITableView,
                   didSelectRowAt indexPath: IndexPath)
    {
        let codeExample = examples[indexPath.row]
        
        if(codeExample.title == "Project Zero")
        {
            targetCodeExample = "Project Zero"
            self.performSegue(withIdentifier: "projectZeroSegue", sender: self)
        }
        else if(codeExample.title == "Temperature Sensor Broadcast")
        {
            targetCodeExample = "MSP432 Thermometer"
            self.performSegue(withIdentifier: "tempSensorSegue", sender: self)
        }
        else if(codeExample.title == "Over the Air Download (OAD)")
        {
            targetCodeExample = "Over the Air Download (OAD)"
            self.performSegue(withIdentifier: "oadSegue", sender: self)
        }
        else if(codeExample.title == "Simple Application Processor")
        {
            targetCodeExample = "Simple Application Processor"
            self.performSegue(withIdentifier: "sapSegue", sender: self)
        }
        else if(codeExample.title == "Sensors BoosterPack")
        {
            targetCodeExample = "Sensors BoosterPack"
            self.performSegue(withIdentifier: "sensorSegue", sender: self)
        }
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat
    {
        return self.view.bounds.height / 8.0
    }
    
    // MARK: Navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?)
    {
        let deviceDiscoveryViewCon: BLEDeviceTableViewController =
            segue.destination as! BLEDeviceTableViewController
    
        deviceDiscoveryViewCon.targetName = targetCodeExample
        deviceDiscoveryViewCon.goSegueIdentifier = segue.identifier!
        deviceDiscoveryViewCon.codeExampleTable = self
    }

}
