/*
 * Copyright (c) 2018, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

import UIKit

class DemoTableViewController: UITableViewController
{
    // MARK: Structure Definition
    struct BLEDemo
    {
        var title = String()
        var marketingString = String()
        var prodURL = String()
        var image = UIImage()
    }
    
    // MARK: Code example list
    let demos =
    [
        BLEDemo(title: "BLE eLock", marketingString: "Capactive touch smart lock with BLE",
                prodURL: "http://www.ti.com/tool/TIDM-1004",
                image: UIImage(named: "lockicon")!),
        BLEDemo(title: "Motor Condition Monitor", marketingString: "BLE enabled motor wear monitor",
                prodURL: "http://www.ti.com/tool/TIDA-01575",
                image: UIImage(named: "gears")!),
    ]
    
    // MARK: Properties:
    @IBOutlet var demoTable: UITableView!
    
    // MARK: Variables
    let cellIdentifier = "BLEDemoTableViewCell"
    var selectedBLEDemo: BLEDemo!
    var targetDemo = ""
    
    // MARK: Functions
    
    
    override func viewDidLoad()
    {
        super.viewDidLoad()
    }

    override func didReceiveMemoryWarning()
    {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    override func tableView(_ tableView: UITableView,
                   numberOfRowsInSection section: Int) -> Int
    {
        return demos.count
    }
    
    override func tableView(_ tableView: UITableView,
                   cellForRowAt indexPath: IndexPath) -> UITableViewCell
    {
        let cell : BLEDemoTableViewCell =
            demoTable.dequeueReusableCell(withIdentifier: cellIdentifier)! as! BLEDemoTableViewCell
        
        let bleDemo = demos[indexPath.row]
        
        cell.marketingString.adjustsFontSizeToFitWidth = true
        cell.demoLabel.adjustsFontSizeToFitWidth = true
        cell.demoLabel.adjustsFontSizeToFitWidth = true
        cell.urlString = bleDemo.prodURL
        cell.demoLabel.text = bleDemo.title
        cell.marketingString.text = bleDemo.marketingString
        cell.demoImage.image = bleDemo.image
        
        return cell
    }
    
    override func tableView(_ tableView: UITableView,
                   didSelectRowAt indexPath: IndexPath)
    {
        let bleDemo = demos[indexPath.row]
        
        if(bleDemo.title == "BLE eLock")
        {
            self.performSegue(withIdentifier: "eLockSegue", sender: self)
        }
        else if(bleDemo.title == "Motor Condition Monitor")
        {
            self.performSegue(withIdentifier: "motorSegue", sender: self)
        }
    }
    
    override func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat
    {
        return 80.0
    }
    
    // MARK: Navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?)
    {
        if(segue.identifier == "safariSegue")
        {
            let webCon: DemoWebViewController =
                segue.destination as! DemoWebViewController
            let webCell: BLEDemoTableViewCell = sender as! BLEDemoTableViewCell
            webCon.urlString = webCell.urlString
        }
        else if(segue.identifier == "motorSegue")
        {
            let motorDeviceSegue: MotorDeviceViewController =
                segue.destination as! MotorDeviceViewController
            motorDeviceSegue.codeExampleTable = self
        }
        else
        {
            let deviceDiscoveryViewCon: BLEDeviceTableViewController =
                segue.destination as! BLEDeviceTableViewController
            
            deviceDiscoveryViewCon.goSegueIdentifier = segue.identifier!
            deviceDiscoveryViewCon.codeExampleTable = self
        }
    }
}
