/*
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

import UIKit

class DeviceSelectionViewController: UIViewController, UITableViewDelegate, UITableViewDataSource
{
    // MARK: - Properties
    @IBOutlet weak var devImage: UIImageView!
    @IBOutlet weak var devTable: UITableView!
    @IBOutlet weak var devLabel: UILabel!
    @IBOutlet weak var devTitle: UILabel!
    
    // MARK: - Variables
    let cellIdentifier = "DeviceSelectorViewCell"
    var referencedController: MainMenuViewController!
    var curSDK: SimpleLinkDevice!
    
    override func viewDidLoad()
    {
        super.viewDidLoad()

        /* Tweaking the GUI a bit */
        devTable.backgroundView = nil
        devTable.isOpaque = false
        devTable.backgroundColor = UIColor.clear
        devTable.delegate = self
        devTable.dataSource = self
        
        devTitle.backgroundColor = UIColor.red
        devTitle.textColor = UIColor.white
        devTitle.layer.cornerRadius = 8.0
        devTitle.clipsToBounds = true
        
        devLabel.layer.shadowOffset = CGSize(width: 0.0,
                                      height: 8.0)
        devLabel.layer.shadowColor = UIColor(displayP3Red: ShadowColor,
                                       green: ShadowColor,
                                       blue: ShadowColor,
                                       alpha: 0.5).cgColor
        devLabel.layer.shadowOpacity = 0.7
        
        devLabel.layer.cornerRadius = 8.0
        devLabel.adjustsFontSizeToFitWidth = true
        navigationController?.navigationBar.titleTextAttributes = [NSAttributedStringKey.foregroundColor: UIColor.white]
        navigationController?.navigationBar.barTintColor = UIColor.red
        
        var curSection: Int = 0
        var curRow: Int = 0
        
        /* Selecting the currently configured device */
        if(curSDK.deviceType == SimpleLinkDeviceType.msp432cc312x)
        {
            curRow = 0
            curSection = 1
        }
        else if(curSDK.deviceType == SimpleLinkDeviceType.msp432cc26xx)
        {
            curRow = 0
            curSection = 0
        }
        
       let selPath = IndexPath(row: curRow, section: curSection)
       devTable.selectRow(at: selPath, animated: false, scrollPosition: UITableViewScrollPosition.top)
       self.tableView(self.devTable, didSelectRowAt: selPath)

    }

    override func didReceiveMemoryWarning()
    {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    // MARK: - Actions
    @IBAction func selectSDKPressed(_ sender: Any)
    {
        let defaults = UserDefaults.standard
        defaults.set(self.curSDK.deviceType.rawValue, forKey: SIMPLELINK_SDK_EXPLORER_KEY)
        referencedController.curDevice = self.curSDK
        referencedController.reloadMenuItems()
        _ = navigationController?.popToViewController(referencedController, animated: true)
    }
    
    @IBAction func cancelSDKSelected(_ sender: Any)
    {
        _ = navigationController?.popToViewController(referencedController, animated: true)
    }


    // MARK: - Table view data source
    func tableView(_ tableView: UITableView,
                   numberOfRowsInSection section: Int) -> Int
    {
        if(section == 0)
        {
            return 1
        }
        else if(section == 1)
        {
            return 1
        }
        else
        {
            return 0
        }
    }
    
    func tableView(_ tableView: UITableView, willDisplayHeaderView view: UIView, forSection section: Int)
    {
        if let view = view as? UITableViewHeaderFooterView
        {
            view.textLabel?.textColor = UIColor.white
            view.backgroundColor = UIColor.clear
            view.backgroundView?.backgroundColor = UIColor.red
            view.backgroundView?.layer.cornerRadius = 8.0
        }
    }
    
    func tableView(_ tableView: UITableView, titleForHeaderInSection section: Int) -> String?
    {
        if(section == 0)
        {
            return "SimpleLink BLE"
        }
        else if(section == 1)
        {
            return "SimpleLink WiFi"
        }
        else
        {
            return "Unknown"
        }
    }
    
    func numberOfSections(in tableView: UITableView) -> Int
    {
        return 2
    }
    
    func tableView(_ tableView: UITableView,
                   cellForRowAt indexPath: IndexPath) -> UITableViewCell
    {
        let cell : DeviceSelectorViewCell =
            devTable.dequeueReusableCell(withIdentifier: cellIdentifier)! as! DeviceSelectorViewCell
        
        var tempDev: SimpleLinkDevice!
        
        if(indexPath.section == 0)
        {
            tempDev = bleDevices[0]
        }
        else if(indexPath.section == 1)
        {
            tempDev = wifiDevices[0]
        }
        
        cell.devLabel.text = tempDev.deviceType.description
        cell.layer.cornerRadius = 5.0
        cell.selectionStyle = UITableViewCellSelectionStyle.blue
        
        return cell
    }
    
    func tableView(_ tableView: UITableView, heightForFooterInSection section: Int) -> CGFloat
    {
        return 10
    }
    
    func tableView(_ tableView: UITableView, viewForFooterInSection section: Int) -> UIView?
    {
        let view = UIView()
        view.backgroundColor = UIColor.clear
        return view
    }
    
    func tableView(_ tableView: UITableView,
                   didSelectRowAt indexPath: IndexPath)
    {
        if(indexPath.section == 0)
        {
            curSDK = bleDevices[0]
        }
        else if(indexPath.section == 1)
        {
            curSDK = wifiDevices[0]
        }

        self.devImage.image = UIImage(named: curSDK.imageIdentifier)
        self.devLabel.text = curSDK.marketingString
        self.devTitle.text = curSDK.deviceType.description
    }
}
