/*
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

import UIKit

class FirmwareSourceViewController: UIViewController, UITableViewDelegate,
    UITableViewDataSource
{
    // MARK: Properties
    @IBOutlet weak var mainTable: UITableView!

    // MARK: Variables
    let cellIdentifier = "FirmwareSourceTableViewCell"
    var referencedController: OADViewController!
    
    // MARK: Source for menu items
    struct MenuItem
    {
        var title = String()
        var image = UIImage()
    }
    
    // MARK: Menu Item List
    let menuItems =
        [
            MenuItem(title: "Box Drive",
                     image: UIImage(named: "box_drive")!),
            MenuItem(title: "Google Drive",
                     image: UIImage(named: "google_drive")!),
            MenuItem(title: "Downloaded Images",
                     image: UIImage(named: "downloadedImages")!),
            
        ]

    override func viewDidLoad()
    {
        super.viewDidLoad()
        mainTable.dataSource = self
        mainTable.delegate = self

        // Do any additional setup after loading the view.
    }

    override func didReceiveMemoryWarning()
    {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    

    // MARK: UITableViewDataSource
    func tableView(_ tableView: UITableView,
                   numberOfRowsInSection section: Int) -> Int
    {
        return menuItems.count
    }
    
    func tableView(_ tableView: UITableView,
                   cellForRowAt indexPath: IndexPath) -> UITableViewCell
    {
        let cell : FirmwareSourceTableViewCell =
            mainTable.dequeueReusableCell(withIdentifier: cellIdentifier)! as! FirmwareSourceTableViewCell
        
        let menuItem = menuItems[indexPath.row]
        
        cell.sourceImage.image = menuItem.image
        
        return cell
    }
    
    // MARK: UITableViewDelegate
    func tableView(_ tableView: UITableView,
                   didSelectRowAt indexPath: IndexPath)
    {
        let menuItem = menuItems[indexPath.row]
        mainTable.deselectRow(at: indexPath, animated: true)
        
        if menuItem.title == "Google Drive"
        {
            self.performSegue(withIdentifier: "googleDriveSegue", sender: self)
        }
        else if (menuItem.title == "Downloaded Images")
        {
            self.performSegue(withIdentifier: "previousDownloadSegue", sender: self)
        }
        else if (menuItem.title == "Box Drive")
        {
            self.performSegue(withIdentifier: "boxContentSegue", sender: self)
        }
    }
    
    // MARK: - Navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?)
    {
        if segue.identifier == "googleDriveSegue"
        {
            let googleDriveController:GoogleDriveViewController =
                segue.destination as! GoogleDriveViewController
            googleDriveController.referencedController = self
        }
        else if segue.identifier == "previousDownloadSegue"
        {
            let previousDownloadSegue: PreviousDownloadTableViewController =
                segue.destination as! PreviousDownloadTableViewController
            previousDownloadSegue.referencedController = self
        }
        else if segue.identifier == "boxContentSegue"
        {
            let boxContentSegue: BoxContentViewController =
                segue.destination as! BoxContentViewController
            boxContentSegue.referencedController = self
        }
    }

}
