/*
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

import UIKit
import CoreBluetooth

class LEDServiceViewController: UIViewController,SwiftHSVColorPickerDelegate
{

    // MARK: Properties
    @IBOutlet weak var colorPicker: SwiftHSVColorPicker!
    @IBOutlet weak var dirLabel: UILabel!

    // MARK: Variables
    var referencedController: ProjectZeroTabBarController!
    var debounceTimer =  Timer()
    var writingValue = false
    var hasLoaded = false
    
    override func viewDidLoad()
    {
        super.viewDidLoad()
        colorPicker.delegate = self
        referencedController = tabBarController as! ProjectZeroTabBarController
        referencedController.ledServiceController = self
        dirLabel.adjustsFontSizeToFitWidth = true
        
        NotificationCenter.default.addObserver(self,
                                               selector: #selector(LEDServiceViewController.viewRotated),
                                               name: NSNotification.Name.UIDeviceOrientationDidChange, object: nil)
    }

    override func didReceiveMemoryWarning()
    {
        super.didReceiveMemoryWarning()
    }
    
    // MARK: UIPickerViewDelegate
    func numberOfComponentsInPickerView(_ pickerView: UIPickerView) -> Int
    {
        return 1
    }
    
    func colorPicker(_ picker: SwiftHSVColorPicker, didChangeColor color: UIColor)
    {
        if(writingValue || self.referencedController == nil || self.referencedController.dataCharData == nil)
        {
            return
        }
        
        colorPicker.colorWheel.isUserInteractionEnabled = false
        colorPicker.brightnessView.isUserInteractionEnabled = false
        
        var r: CGFloat = 0, g: CGFloat = 0, b: CGFloat = 0, a: CGFloat = 0
        color.getRed(&r, green: &g, blue: &b, alpha: &a)
        
        /* Making sure we are actually a number */
        if r.isNaN || r < 0
        {
            r = 0.0
        }
        
        if g.isNaN || g < 0
        {
            g = 0.0
        }
        
        if b.isNaN || b < 0
        {
            b = 0.0
        }
        
        /* Converting into UInt8 and sending to the device */
        let rInt = UInt8(r*255), gInt = UInt8(g*255), bInt = UInt8(b*255)
        let writeArray: [UInt8] = [rInt,gInt,bInt]
        referencedController.btDevice.writeValue(Data(writeArray),
                                                      for: referencedController.ledCharLED0,
                                                      type: CBCharacteristicWriteType.withResponse)
        writingValue = true
        debounceTimer = Timer.scheduledTimer(timeInterval: 0.05,
                                             target: self,
                                             selector: #selector(debounceAction),
                                             userInfo: nil, repeats: false)
    }
    
    @objc func debounceAction()
    {
        writingValue = false
        colorPicker.colorWheel.isUserInteractionEnabled = true
        colorPicker.brightnessView.isUserInteractionEnabled = true
    }
    
    override func viewDidAppear(_ animated: Bool)
    {
        DispatchQueue.main.async
        {
            self.colorPicker.setViewColor(UIColor.red)
            
            if(self.referencedController.dataCharData != nil)
            {
                self.referencedController.referencedController.connectionCon.dismiss(animated: true, completion: nil)
                self.referencedController.referencedController.timeoutTimer.invalidate()
            }
            
            self.hasLoaded = true
        }
    }
    
    @objc func viewRotated()
    {
        if(self.colorPicker == nil)
        {
            return
        }
        
        DispatchQueue.main.async
        {
            self.colorPicker.setViewColor(self.colorPicker.color)
        }
    }

}
