/*
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

import UIKit

class MainMenuViewController: UIViewController, UITableViewDelegate,
                                UITableViewDataSource
{
    // MARK: Properties
    @IBOutlet weak var mainTable: UITableView!
    @IBOutlet weak var simpleStarterProperty: UILabel!
    @IBOutlet weak var curConfigProperty: UILabel!
    @IBOutlet weak var configContainerView: UIView!
    @IBOutlet weak var curDevLabel: UILabel!
    @IBOutlet weak var curDevImage: UIImageView!
    @IBOutlet weak var configStack: UIStackView!
    
    // MARK: Menu Item List
    let menuItemsBLEPlugin =
        [
            MenuItem(title: "Profile Examples",
                     image: UIImage(named: "examples")!),
            MenuItem(title: "Demos/Designs",
                     image: UIImage(named: "demos")!),
            MenuItem(title: "User's Guide",
                     image: UIImage(named: "userguide")!),
            MenuItem(title: "Support",
                     image: UIImage(named: "support")!),
        ]
    
    let menuItemsBLE =
        [
            MenuItem(title: "Profile Examples",
                     image: UIImage(named: "examples")!),
            MenuItem(title: "User's Guide",
                     image: UIImage(named: "userguide")!),
            MenuItem(title: "Support",
                     image: UIImage(named: "support")!),
        ]
    
    let menuItemsWifiPlugin =
        [
            MenuItem(title: "Provisioning",
                     image: UIImage(named: "wifi_big")!),
            MenuItem(title: "Provisioned Devices",
                     image: UIImage(named: "examples")!),
            MenuItem(title: "User's Guide",
                     image: UIImage(named: "userguide")!),
            MenuItem(title: "Support",
                     image: UIImage(named: "support")!),
            
        ]
    
    
    let menuItemsWifi =
        [
            MenuItem(title: "User's Guide",
                     image: UIImage(named: "userguide")!),
            MenuItem(title: "Provisioning",
                     image: UIImage(named: "wifi_big")!),
            MenuItem(title: "Firmware Update",
                     image: UIImage(named: "oad")!),
            MenuItem(title: "Support",
                     image: UIImage(named: "support")!),
        ]
    
    
    let menuItemsSub1 =
        [
            MenuItem(title: "User's Guide",
                     image: UIImage(named: "userguide")!),
            MenuItem(title: "Support",
                     image: UIImage(named: "support")!),
        ]
    
    
    let menuItemsSub1Plugin =
        [
            MenuItem(title: "User's Guide",
                     image: UIImage(named: "userguide")!),
            MenuItem(title: "Support",
                     image: UIImage(named: "support")!),
        ]
    
    
    // MARK: Variables
    let cellIdentifier = "PluginTableViewCell"
    var curDevice: SimpleLinkDevice = bleDevices[0]
    var curMenu: [MenuItem] = []
    
    // MARK: Structure Definition
    struct MenuItem
    {
        var title = String()
        var image = UIImage()
    }
    
    override func viewDidLoad()
    {
        super.viewDidLoad()
        
        /* Customizing the GUI a bit */
        mainTable.dataSource = self
        mainTable.delegate = self
        curConfigProperty.layer.cornerRadius = 8.0
        curConfigProperty.clipsToBounds = true
        simpleStarterProperty.layer.cornerRadius = 8.0
        simpleStarterProperty.clipsToBounds = true
        configContainerView.layer.borderColor = UIColor.lightGray.cgColor
        configContainerView.layer.cornerRadius = 8.0
        configContainerView.layer.borderWidth = 1.0
        configContainerView.clipsToBounds = true
        curDevLabel.adjustsFontSizeToFitWidth = true
        
        /* Adding observer to rotate view */
        NotificationCenter.default.addObserver(self,
                                               selector: #selector(MainMenuViewController.viewRotated),
                                               name: NSNotification.Name.UIDeviceOrientationDidChange, object: nil)
        
        let defaults = UserDefaults.standard
        let curSDKRaw = defaults.object(forKey: SIMPLELINK_SDK_EXPLORER_KEY) as? Int
        
        if(curSDKRaw != nil)
        {
            let curSDK = SimpleLinkDeviceType(rawValue: curSDKRaw!)
            
            if(curSDK == SimpleLinkDeviceType.cc2640r2)
            {
                curDevice = bleDevices[0]
            }
            else if(curSDK == SimpleLinkDeviceType.msp432cc26xx)
            {
                curDevice = bleDevices[0]
            }
            else if(curSDK == SimpleLinkDeviceType.cc3220)
            {
                curDevice = wifiDevices[0]
            }
            else if(curSDK == SimpleLinkDeviceType.msp432cc312x)
            {
                curDevice = wifiDevices[0]
            }
        }
        
        /* Intializing the current device */
        reloadMenuItems()
    }
    
    override func viewWillDisappear(_ animated: Bool)
    {
        self.navigationController?.setNavigationBarHidden(false,
                                                          animated: false)
        super.viewWillDisappear(animated)
    }
    
    override func viewWillAppear(_ animated: Bool)
    {
        self.navigationController?.setNavigationBarHidden(true,
                                                          animated: false)
        super.viewWillAppear(animated)
    }

    override func didReceiveMemoryWarning()
    {
        super.didReceiveMemoryWarning()
    }
    
    // MARK: UITableViewDataSource
    func tableView(_ tableView: UITableView,
                   numberOfRowsInSection section: Int) -> Int
        
    {
        return curMenu.count
    }
    
    func tableView(_ tableView: UITableView,
                   cellForRowAt indexPath: IndexPath) -> UITableViewCell
    {
        let cell : PluginTableViewCell =
            mainTable.dequeueReusableCell(withIdentifier: cellIdentifier)! as! PluginTableViewCell
        
        let menuItem = curMenu[indexPath.row]
        
        cell.titleText.text = menuItem.title
        cell.exampleImage.image = menuItem.image
        
        return cell
    }
    
    // MARK: UITableViewDelegate
    func tableView(_ tableView: UITableView,
                   didSelectRowAt indexPath: IndexPath)
    {
        let menuItem = curMenu[indexPath.row]
        
        if menuItem.title == "Profile Examples"
        {
            self.performSegue(withIdentifier: "codeExampleSegue", sender: self)
        }
        else if(menuItem.title == "Demos/Designs")
        {
            self.performSegue(withIdentifier: "demosSegue", sender: self)
        }
        else if(menuItem.title == "Support")
        {
            UIApplication.shared.open(URL(string: curDevice.supportURL)!, options: [:], completionHandler: nil)
        }
        else if(menuItem.title == "User's Guide")
        {
            UIApplication.shared.open(URL(string: curDevice.userGuideURL)!, options: [:], completionHandler: nil)
        }
        else if(menuItem.title == "Provisioning")
        {
            self.performSegue(withIdentifier: "wifiProvisionSegue", sender: self)
        }
        else if(menuItem.title == "Provisioned Devices")
        {
            self.performSegue(withIdentifier: "prevDeviceSegue", sender: self)
        }
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat
    {
        return self.view.bounds.height / 7.0
    }
    
    // MARK: Actions
    @IBAction func privacyPolicy(_ sender: Any)
    {
        UIApplication.shared.open(URL(string: "http://www.ti.com/corp/docs/legal/privacy.shtml")!,
                                  options: [:], completionHandler: nil)
    }
    
    @IBAction func termsOfUse(_ sender: Any)
    {
        UIApplication.shared.open(URL(string: "http://www.ti.com/corp/docs/legal/termsofuse.shtml")!,
                                  options: [:], completionHandler: nil)
    }
    
    // MARK: Helper functions
    func reloadMenuItems()
    {
        if(self.curDevice.deviceType == SimpleLinkDeviceType.msp432cc26xx)
        {
            self.curMenu = self.menuItemsBLEPlugin
            self.curDevImage.image = UIImage(named: "ble_little")
            
        }
        else if(self.curDevice.deviceType == SimpleLinkDeviceType.cc13x0)
        {
            self.curMenu = self.menuItemsSub1
            self.curDevImage.image = UIImage(named: "little_sub1")
        }
        else if(self.curDevice.deviceType == SimpleLinkDeviceType.cc2640r2)
        {
            self.curMenu = self.menuItemsBLE
            self.curDevImage.image = UIImage(named: "ble_little")
        }
        else if(self.curDevice.deviceType == SimpleLinkDeviceType.cc3220)
        {
            self.curMenu = self.menuItemsWifi
            self.curDevImage.image = UIImage(named: "simplelinkwifi")
        }
        else if(self.curDevice.deviceType == SimpleLinkDeviceType.msp432cc312x)
        {
            self.curMenu = self.menuItemsWifiPlugin
            self.curDevImage.image = UIImage(named: "simplelinkwifi")
        }
        
        self.curDevLabel.text = curDevice.description
        
        DispatchQueue.main.async
        {
            self.mainTable.reloadData()
        }
    }
    
    // MARK: Navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?)
    {
        if(segue.identifier == "showDeviceConfigSegue")
        {
            let devSelectController: DeviceSelectionViewController =
                segue.destination as! DeviceSelectionViewController
            
            devSelectController.curSDK = self.curDevice
            devSelectController.referencedController = self
        }
    }
    
    @objc func viewRotated()
    {
        DispatchQueue.main.async
            {
                if UIDeviceOrientationIsPortrait(UIDevice.current.orientation)
                {
                    self.configStack.isHidden = false
                }
                
                if UIDeviceOrientationIsLandscape(UIDevice.current.orientation)
                {
                    self.configStack.isHidden = true
                }
        }
    }
}
