/*
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

import UIKit

class PreviousDownloadTableViewController: UITableViewController
{
    // MARK: - Variables
    let cellIdentifier = "FirmwareImageViewCell"
    var referencedController: FirmwareSourceViewController!
    
    // MARK: - Properties
    @IBOutlet var firmwareImageTable: UITableView!

    override func viewDidLoad()
    {
        super.viewDidLoad()

    }

    override func didReceiveMemoryWarning()
    {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    // MARK: - Table view data source

    override func numberOfSections(in tableView: UITableView) -> Int
    {
        return 1
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int
    {
            return referencedController.referencedController.firmwareImages.count
    }
    
    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell
    {
        let cell : PreviousDownloadTableViewCell =
            firmwareImageTable.dequeueReusableCell(withIdentifier: cellIdentifier)! as! PreviousDownloadTableViewCell
        
        let firmwareImage = referencedController.referencedController.firmwareImages[indexPath.row]
        
        cell.firmwareLabel.text = firmwareImage.getTableLabel()
        
        return cell
    }
    
    override func tableView(_ tableView: UITableView, commit editingStyle: UITableViewCellEditingStyle, forRowAt indexPath: IndexPath)
    {
        if editingStyle == .delete
        {
            let imageToDelete: ImportedFirmwareImage! =
                referencedController.referencedController.firmwareImages[(indexPath as NSIndexPath).row]
            
            /* If they are deleting the current translation, we have to reset the
             view */
            if referencedController.referencedController.curFirmware != nil &&
                referencedController.referencedController.curFirmware.isEqual(imageToDelete)
            {
                referencedController.referencedController.firmwareNameLabel.text = "Touch to Select Firmware"
            }
            
            referencedController.referencedController.firmwareImages.remove(at: (indexPath as NSIndexPath).row)
            referencedController.referencedController.saveFirmwareImages()
            firmwareImageTable.deleteRows(at: [indexPath], with: .fade)
        }
    }
    
    override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath)
    {
        referencedController.referencedController.curFirmware =
            referencedController.referencedController.firmwareImages[(indexPath as NSIndexPath).row]
        referencedController.referencedController.firmwareNameLabel.text =
            referencedController.referencedController.curFirmware.getTableLabel()
        _ = navigationController?.popToViewController(referencedController.referencedController, animated: true)
    }

}
