/*
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

import UIKit
import NetworkExtension

class ProvisionedDeviceViewController: UIViewController
{
    // MARK: Properties:
    @IBOutlet weak var dateLabel: UILabel!
    @IBOutlet weak var versionLabel: UILabel!
    @IBOutlet weak var confLabel: UILabel!
    @IBOutlet weak var commonLabel: UILabel!
    @IBOutlet weak var directLabel: UILabel!
    @IBOutlet weak var logArea: UITextView!
    @IBOutlet weak var slInfoLabel: UILabel!
    @IBOutlet weak var routerLabel: UILabel!
    @IBOutlet weak var openButton: UIButton!
    @IBOutlet weak var scrollView: UIScrollView!
    @IBOutlet weak var backButton: UIButton!
    @IBOutlet weak var devNameLabel: UILabel!
    @IBOutlet weak var stackView: UIStackView!
    @IBOutlet weak var directTitle: UILabel!
    @IBOutlet weak var commonTitle: UILabel!
    @IBOutlet weak var deviceTitle: UILabel!
    @IBOutlet weak var versionTitle: UILabel!
    @IBOutlet weak var routerTitle: UILabel!
    
    // MARK: Variables
    var curDevice: ProvisionedWiFiDevice!
    
    // MARK: Actions
    @IBAction func openPressed(_ sender: Any)
    {
        var currentDevices = WiFiProvisioningManager.currentSSIDs()
        let routerName = curDevice.wifiSettings.routerName
        
        if(currentDevices.count == 0 || currentDevices[0] != curDevice.wifiSettings.routerName)
        {
            let connectAlert = UIAlertController(title: "Connect to " + routerName + "?",
                                                 message: "You are not currently connected to the common network" +
                                                            " that the device was provisioned for (" + routerName + ")"
                                                    + ". Connect now?", preferredStyle: UIAlertControllerStyle.alert)
            
            connectAlert.addAction(UIAlertAction(title: "No", style: .default, handler:
                { (action: UIAlertAction!) in
                    self.openDeviceURL()
            }))
            
            connectAlert.addAction(UIAlertAction(title: "Yes", style: .default, handler:
            { (action: UIAlertAction!) in
                var routerDev: NEHotspotConfiguration?
                
                /* Creating a configuration for the router we want to connect to */
                if(self.curDevice.wifiSettings.devSecType == WiFiSecurityType.None)
                {
                    routerDev = NEHotspotConfiguration(ssid: routerName)
                }
                else if(self.curDevice.wifiSettings.devSecType == WiFiSecurityType.WPA)
                {
                    routerDev = NEHotspotConfiguration(ssid: routerName,
                                                           passphrase: self.curDevice.wifiSettings.routerKey,
                                                           isWEP: false)
                }
                else if(self.curDevice.wifiSettings.devSecType == WiFiSecurityType.WEP)
                {
                    routerDev = NEHotspotConfiguration(ssid: routerName,
                                                           passphrase: self.curDevice.wifiSettings.routerKey,
                                                           isWEP: true)
                }
                
                /* Grabs the shared configuration manager and applies the connection of the SimpleLink device.*/
                NEHotspotConfigurationManager.shared.apply(routerDev!, completionHandler:
                    { (error) in
                        
                        /* In the completion handler we want to see if the current device is the one we actually tried to
                         connect to. If not we should return an error. */
                        let connectedNet = WiFiProvisioningManager.currentSSIDs()
                        if(error == nil && connectedNet.count > 0 && connectedNet[0] == routerName)
                        {
                            self.openDeviceURL()
                        }
                })
                
            }))
            
            present(connectAlert, animated: true, completion: nil)
        }
        else
        {
            openDeviceURL()
        }
    }
    
    @IBAction func backPressed(_ sender: Any)
    {
        self.dismiss(animated: true, completion: nil)
    }
    
    // MARK: Functions
    func openDeviceURL()
    {
        UIApplication.shared.open(URL(string: curDevice.devURL)!, options: [:], completionHandler: nil)
    }
    
    override func viewDidLoad()
    {
        super.viewDidLoad()
        openButton.clipsToBounds = true
        openButton.layer.cornerRadius = 8.0
        backButton.clipsToBounds = true
        backButton.layer.cornerRadius = 8.0
        self.devNameLabel.text = curDevice.wifiSettings.devName
        self.dateLabel.text = curDevice.dateAdded
        self.confLabel.text = curDevice.confirmMethod
        
        if(curDevice.confirmMethod.range(of: "BLE") != nil)
        {
            openButton.isHidden = true
        }
        
        if(curDevice.commonURL.isEmpty == true)
        {
            self.commonLabel.isHidden = true
            self.commonTitle.isHidden = true
        }
        else
        {
            self.commonLabel.text = curDevice.commonURL
        }
        
        if(curDevice.directURL.isEmpty == true)
        {
            self.directLabel.isHidden = true
            self.directTitle.isHidden = true
        }
        else
        {
            self.directLabel.text = curDevice.directURL
        }
        self.logArea.text = curDevice.provisionLog
        
        if(curDevice.devVersion.isEmpty == true)
        {
            self.versionLabel.isHidden = true
            self.versionTitle.isHidden = true
        }
        else
        {
            self.versionLabel.text = curDevice.devVersion
        }
        logArea.clipsToBounds = true
        logArea.layer.cornerRadius = 8.0
        
        self.routerLabel.text = curDevice.wifiSettings.routerName
        
        if(curDevice.wifiSettings.simplelinkName.isEmpty == true)
        {
            deviceTitle.isHidden = true
            slInfoLabel.isHidden = true
        }
        
        if(curDevice.wifiSettings.routerSecType == .None)
        {
            self.routerLabel.text = curDevice.wifiSettings.routerName + " - Open"
        }
        else if(curDevice.wifiSettings.routerSecType == .WEP)
        {
            self.routerLabel.text = curDevice.wifiSettings.routerName + " - WEP (" +
                curDevice.wifiSettings.routerKey + ")"
            
        }
        else if(curDevice.wifiSettings.routerSecType == .WPA)
        {
            self.routerLabel.text = curDevice.wifiSettings.routerName + " - WPA/WPA2 (" +
                curDevice.wifiSettings.routerKey + ")"
        }
        
        if(curDevice.wifiSettings.devSecType == .None)
        {
            self.slInfoLabel.text = curDevice.wifiSettings.simplelinkName + " - Open"
        }
        else if(curDevice.wifiSettings.devSecType == .WEP)
        {
            self.slInfoLabel.text = curDevice.wifiSettings.simplelinkName + " - WEP (" +
                curDevice.wifiSettings.simplelinkKey + ")"
            
        }
        else if(curDevice.wifiSettings.devSecType == .WPA)
        {
            self.slInfoLabel.text = curDevice.wifiSettings.simplelinkName + " - WPA/WPA2 (" +
                curDevice.wifiSettings.simplelinkKey + ")"
        }
        
        
        var newSize = stackView.bounds.size
        newSize.height = newSize.height * 1.5
        scrollView.contentSize = newSize
    }

    override func didReceiveMemoryWarning()
    {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

}
