/*
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

import UIKit

class ProvisionedDevicesTableViewController: UITableViewController
{
    // MARK: Variables
    var provisionedDevices = [ProvisionedWiFiDevice]()
    let cellIdentifier = "ProvisionedDeviceTableViewCell"
    var selectedDev: ProvisionedWiFiDevice!
    
    // MARK: Functions
    override func viewDidLoad()
    {
        super.viewDidLoad()
        
        /* Loading up the previously provisioned devices */
        provisionedDevices = loadProvisionedDevices()!
    }

    override func didReceiveMemoryWarning()
    {
        super.didReceiveMemoryWarning()
    }

    // MARK: - Table view data source
    override func numberOfSections(in tableView: UITableView) -> Int
    {
        return 1
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int
    {
        return provisionedDevices.count
    }

    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell
    {
        let cell : ProvisionedDeviceTableViewCell =
            tableView.dequeueReusableCell(withIdentifier: cellIdentifier)! as! ProvisionedDeviceTableViewCell
        
        let provisionDevice = provisionedDevices[indexPath.row]
        cell.devLabel.text = provisionDevice.getTableLabel()
        cell.devImage.image = UIImage(named: "wifi_little")!
        cell.devLabel.adjustsFontSizeToFitWidth = true
        
        return cell
    }
    
    override func tableView(_ tableView: UITableView, commit editingStyle: UITableViewCellEditingStyle, forRowAt indexPath: IndexPath)
    {
        if editingStyle == .delete
        {
            /* If they are deleting the current translation, we have to reset the
             view */
            provisionedDevices.remove(at: (indexPath as NSIndexPath).row)
            saveProvisionedDevices()
            tableView.deleteRows(at: [indexPath], with: .fade)
        }
    }
    
    override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath)
    {
        selectedDev = provisionedDevices[indexPath.row]
        self.performSegue(withIdentifier: "showDev", sender: self)
    }
    
    override func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat
    {
        return 70.0
    }
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?)
    {
        if segue.identifier == "showDev"
        {
            let provDevController: ProvisionedDeviceViewController = segue.destination
                as! ProvisionedDeviceViewController
            provDevController.curDevice = selectedDev
        }
    }
    
    // MARK: - NSCoding
    func loadProvisionedDevices() -> [ProvisionedWiFiDevice]?
    {
        var provisionedDevices = NSKeyedUnarchiver.unarchiveObject(withFile: ProvisionedWiFiDevice.ArchiveURL.path)
            as? [ProvisionedWiFiDevice]
        
        if(provisionedDevices == nil)
        {
            provisionedDevices = [ProvisionedWiFiDevice]()
        }
        
        return provisionedDevices
    }
    
    func saveProvisionedDevices()
    {
        NSKeyedArchiver.archiveRootObject(provisionedDevices, toFile: ProvisionedWiFiDevice.ArchiveURL.path)
    }
}
