/*
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

import UIKit

class ProvisionedWiFiDevice: NSObject, NSCoding
{
    // MARK: - Member variables
    var directURL: String!
    var commonURL: String!
    var wifiSettings: WiFiSettings!
    var confirmMethod: String!
    var dateAdded: String!
    var devURL: String!
    var devVersion: String!
    var provisionLog: String!

    // MARK: Types
    struct PropertyKey
    {
        static let directURL = "directURL"
        static let commonURL = "commonURL"
        static let confirmationMethod = "confirmMethod"
        static let dateAdded = "dateAdded"
        static let deviceURL = "deviceURL"
        static let deviceName = "deviceName"
        static let routerSSID = "routerSSID"
        static let routerSecType = "routerSec"
        static let routerSecKey = "routerKey"
        static let slName = "slName"
        static let slSecType = "slSecType"
        static let slSecKey = "slSecKey"
        static let devVersion = "devVersion"
        static let provLog = "provLog"
    }
    
    override init()
    {
        super.init()
    }
    
    init(dirURL: String, commonURL: String, wifiSettings: WiFiSettings,confirm: String, dateAdded: String, url: String,
         devVer: String!, provLog: String, settings: WiFiSettings)
    {
        super.init()
        
        self.directURL = dirURL
        self.commonURL = commonURL
        self.wifiSettings = wifiSettings
        self.confirmMethod = confirm
        self.dateAdded = dateAdded
        self.devURL = url
        self.devVersion = devVer
        self.provisionLog = provLog
    }
    
    //MARK: NSCoding
    func encode(with aCoder: NSCoder)
    {
        aCoder.encode(directURL, forKey: PropertyKey.directURL)
        aCoder.encode(commonURL, forKey: PropertyKey.commonURL)
        aCoder.encode(confirmMethod, forKey: PropertyKey.confirmationMethod)
        aCoder.encode(dateAdded, forKey: PropertyKey.dateAdded)
        aCoder.encode(devURL, forKey: PropertyKey.deviceURL)
        aCoder.encode(devVersion, forKey: PropertyKey.devVersion)
        aCoder.encode(wifiSettings.routerName, forKey: PropertyKey.routerSSID)
        aCoder.encode(wifiSettings.routerSecType.rawValue, forKey: PropertyKey.routerSecType)
        aCoder.encode(wifiSettings.routerKey, forKey: PropertyKey.routerSecKey)
        aCoder.encode(wifiSettings.simplelinkName, forKey: PropertyKey.slName)
        aCoder.encode(wifiSettings.devSecType.rawValue, forKey: PropertyKey.slSecType)
        aCoder.encode(wifiSettings.simplelinkKey, forKey: PropertyKey.slSecKey)
        aCoder.encode(wifiSettings.devName, forKey: PropertyKey.deviceName)
        aCoder.encode(provisionLog, forKey: PropertyKey.provLog)
    }
    
    required convenience init?(coder aDecoder: NSCoder)
    {
        let decodedDirectURL = aDecoder.decodeObject(forKey: PropertyKey.directURL) as! String
        let decodedCommonURL = aDecoder.decodeObject(forKey: PropertyKey.commonURL) as! String
        let decodedConfirmMethod = aDecoder.decodeObject(forKey: PropertyKey.confirmationMethod) as! String
        let decodedDate = aDecoder.decodeObject(forKey: PropertyKey.dateAdded) as! String
        let decodedDevURL = aDecoder.decodeObject(forKey: PropertyKey.deviceURL) as! String
        let decodedDevName = aDecoder.decodeObject(forKey: PropertyKey.deviceName) as! String
        let decodedRouterName = aDecoder.decodeObject(forKey: PropertyKey.routerSSID) as! String
        let decodedRouterSecType = aDecoder.decodeObject(forKey: PropertyKey.routerSecType) as! String
        let decodedRouterKey = aDecoder.decodeObject(forKey: PropertyKey.routerSecKey) as! String
        let decodedSLName = aDecoder.decodeObject(forKey: PropertyKey.slName) as! String
        let decodedSLSecType = aDecoder.decodeObject(forKey: PropertyKey.slSecType) as! String
        let decodedSLKey = aDecoder.decodeObject(forKey: PropertyKey.slSecKey) as! String
        let decodedVersion = aDecoder.decodeObject(forKey: PropertyKey.devVersion) as! String
        let decodedLog = aDecoder.decodeObject(forKey: PropertyKey.provLog) as! String
        
        let wifiSettings = WiFiSettings(routerName: decodedRouterName, routerKey: decodedRouterKey,
                                        simplelinkName: decodedSLName, simplelinkKey: decodedSLKey,
                                        devName: decodedDevName,
                                        routerSecType: WiFiSecurityType(rawValue: decodedRouterSecType)!,
                                        devSecType: WiFiSecurityType(rawValue: decodedSLSecType)!)
        
        
        self.init(dirURL: decodedDirectURL, commonURL: decodedCommonURL, wifiSettings: wifiSettings,
                  confirm: decodedConfirmMethod, dateAdded: decodedDate, url: decodedDevURL, devVer: decodedVersion,
                  provLog: decodedLog ,settings: wifiSettings)
    }
    
    // MARK: - Functions
    func getTableLabel() -> String
    {
        return "\(self.wifiSettings.devName) - \(self.dateAdded!)"
    }
    
    // MARK: Archiving Paths:
    static let DocumentsDirectory = FileManager().urls(for: .documentDirectory, in: .userDomainMask).first!
    static let ArchiveURL = DocumentsDirectory.appendingPathComponent("provisionedWiFi")
    
}
