import UIKit
import CoreBluetooth

class SensorTabBarController: UITabBarController, CBPeripheralDelegate
{
    
    // MARK: Variables
    var referencedController: BLEDeviceTableViewController!
    var sensorOverViewController: SensorCollectionViewController!
    var irTempGraphController: IRTempViewController!
    var opticalGraphController: OpticalViewController!
    var humidityGraphController: HumidityViewController!
    var barometerGraphController: BarometerViewController!
    var gyrGraphController: GyrViewController!
    var accGraphController: AccViewController!
    var magGraphController: MagViewController!
    var btDevice: CBPeripheral!
    
    // MARK Device UUIDs
    let TI_SN_IRTEMP_SERVICE_UUID = CBUUID(string: "F000AA00-0451-4000-B000-000000000000")
    let TI_SN_IRTEMP_DATA_CHAR_UUID = CBUUID(string: "F000AA01-0451-4000-B000-000000000000")
    let TI_SN_IRTEMP_CONF_CHAR_UUID = CBUUID(string: "F000AA02-0451-4000-B000-000000000000")
    let TI_SN_IRTEMP_PERI_CHAR_UUID = CBUUID(string: "F000AA03-0451-4000-B000-000000000000")
    let TI_SN_HUMIDITY_SERVICE_UUID = CBUUID(string: "F000AA20-0451-4000-B000-000000000000")
    let TI_SN_HUMIDITY_DATA_CHAR_UUID = CBUUID(string: "F000AA21-0451-4000-B000-000000000000")
    let TI_SN_HUMIDITY_CONF_CHAR_UUID = CBUUID(string: "F000AA22-0451-4000-B000-000000000000")
    let TI_SN_HUMIDITY_PERI_CHAR_UUID = CBUUID(string: "F000AA23-0451-4000-B000-000000000000")
    let TI_SN_BAROMETER_SERVICE_UUID = CBUUID(string: "F000AA40-0451-4000-B000-000000000000")
    let TI_SN_BAROMETER_DATA_CHAR_UUID = CBUUID(string: "F000AA41-0451-4000-B000-000000000000")
    let TI_SN_BAROMETER_CONF_CHAR_UUID = CBUUID(string: "F000AA42-0451-4000-B000-000000000000")
    let TI_SN_BAROMETER_PERI_CHAR_UUID = CBUUID(string: "F000AA44-0451-4000-B000-000000000000")
    let TI_SN_OPTIC_SERVICE_UUID = CBUUID(string: "F000AA70-0451-4000-B000-000000000000")
    let TI_SN_OPTIC_DATA_CHAR_UUID = CBUUID(string: "F000AA71-0451-4000-B000-000000000000")
    let TI_SN_OPTIC_CONF_CHAR_UUID = CBUUID(string: "F000AA72-0451-4000-B000-000000000000")
    let TI_SN_OPTIC_PERI_CHAR_UUID = CBUUID(string: "F000AA73-0451-4000-B000-000000000000")
    let TI_SN_MOVE_SERVICE_UUID = CBUUID(string: "F000AA80-0451-4000-B000-000000000000")
    let TI_SN_MOVE_DATA_CHAR_UUID = CBUUID(string: "F000AA81-0451-4000-B000-000000000000")
    let TI_SN_MOVE_CONF_CHAR_UUID = CBUUID(string: "F000AA82-0451-4000-B000-000000000000")
    let TI_SN_MOVE_PERI_CHAR_UUID = CBUUID(string: "F000AA83-0451-4000-B000-000000000000")
    
    // MARK: Bluetooth Characteristics
    var tempDataChar: CBCharacteristic!
    var tempConfChar: CBCharacteristic!
    var tempPeriChar: CBCharacteristic!
    var humDataChar: CBCharacteristic!
    var humConfChar: CBCharacteristic!
    var humPeriChar: CBCharacteristic!
    var baroDataChar: CBCharacteristic!
    var baroConfChar: CBCharacteristic!
    var baroPeriChar: CBCharacteristic!
    var optDataChar: CBCharacteristic!
    var optConfChar: CBCharacteristic!
    var optPeriChar: CBCharacteristic!
    var movDataChar: CBCharacteristic!
    var movConfChar: CBCharacteristic!
    var movPeriChar: CBCharacteristic!

    override func viewDidLoad()
    {
        super.viewDidLoad()

        /* Adding observer to rotate view */
        NotificationCenter.default.addObserver(self,
                                               selector: #selector(SensorTabBarController.viewRotated),
                                               name: NSNotification.Name.UIDeviceOrientationDidChange, object: nil)
    }
    
    @objc func viewRotated()
    {
        DispatchQueue.main.async
            {
                if UIDeviceOrientationIsPortrait(UIDevice.current.orientation)
                {
                    //self.tabBar.isHidden = false
                    //self.navigationController?.navigationBar.isHidden = false
                }
                else if UIDeviceOrientationIsLandscape(UIDevice.current.orientation)
                {
                    //self.tabBar.isHidden = true
                    //self.navigationController?.navigationBar.isHidden = true
                }
        }
    }

    override func didReceiveMemoryWarning()
    {
    
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    // MARK: CBPeripheralDelegate
    func peripheral(_ peripheral: CBPeripheral,
                    didDiscoverServices error: Error?)
    {
        for service in peripheral.services!
        {
            let thisService = service as CBService
            peripheral.discoverCharacteristics(nil, for: thisService)
        }
    }
    
    func peripheral(_ peripheral: CBPeripheral,
                    didDiscoverCharacteristicsFor service: CBService,
                    error: Error?)
    {
        if(service.uuid == TI_SN_IRTEMP_SERVICE_UUID)
        {
            for characteristic in service.characteristics!
            {
                if characteristic.uuid == TI_SN_IRTEMP_DATA_CHAR_UUID
                {
                    tempDataChar = characteristic
                }
                else if(characteristic.uuid == TI_SN_IRTEMP_CONF_CHAR_UUID)
                {
                    tempConfChar = characteristic
                }
                else if(characteristic.uuid == TI_SN_IRTEMP_PERI_CHAR_UUID)
                {
                    tempPeriChar = characteristic
                }
                else
                {
                    print("SenseHub: Unknown characteristic UUID: \(characteristic.uuid.uuidString)")
                    continue
                }
            }
        }
        else if(service.uuid == TI_SN_HUMIDITY_SERVICE_UUID)
        {
            for characteristic in service.characteristics!
            {
                if characteristic.uuid == TI_SN_HUMIDITY_DATA_CHAR_UUID
                {
                    humDataChar = characteristic
                }
                else if(characteristic.uuid == TI_SN_HUMIDITY_CONF_CHAR_UUID)
                {
                    humConfChar = characteristic
                }
                else if(characteristic.uuid == TI_SN_HUMIDITY_PERI_CHAR_UUID)
                {
                    humPeriChar = characteristic
                }
                else
                {
                    print("SenseHub: Unknown characteristic UUID: \(characteristic.uuid.uuidString)")
                    continue
                }
            }
        }
        else if(service.uuid == TI_SN_BAROMETER_SERVICE_UUID)
        {
            for characteristic in service.characteristics!
            {
                if characteristic.uuid == TI_SN_BAROMETER_DATA_CHAR_UUID
                {
                    baroDataChar = characteristic
                }
                else if(characteristic.uuid == TI_SN_BAROMETER_CONF_CHAR_UUID)
                {
                    baroConfChar = characteristic
                }
                else if(characteristic.uuid == TI_SN_BAROMETER_PERI_CHAR_UUID)
                {
                    baroPeriChar = characteristic
                }
                else
                {
                    print("SenseHub: Unknown characteristic UUID: \(characteristic.uuid.uuidString)")
                    continue
                }
            }
        }
        else if(service.uuid == TI_SN_OPTIC_SERVICE_UUID)
        {
            for characteristic in service.characteristics!
            {
                if characteristic.uuid == TI_SN_OPTIC_DATA_CHAR_UUID
                {
                    optDataChar = characteristic
                }
                else if(characteristic.uuid == TI_SN_OPTIC_CONF_CHAR_UUID)
                {
                    optConfChar = characteristic
                }
                else if(characteristic.uuid == TI_SN_OPTIC_PERI_CHAR_UUID)
                {
                    optPeriChar = characteristic
                }
                else
                {
                    print("SenseHub: Unknown characteristic UUID: \(characteristic.uuid.uuidString)")
                    continue
                }
            }
        }
        else if(service.uuid == TI_SN_MOVE_SERVICE_UUID)
        {
            for characteristic in service.characteristics!
            {
                if characteristic.uuid == TI_SN_MOVE_DATA_CHAR_UUID
                {
                    movDataChar = characteristic
                }
                else if(characteristic.uuid == TI_SN_MOVE_CONF_CHAR_UUID)
                {
                    movConfChar = characteristic
                }
                else if(characteristic.uuid == TI_SN_MOVE_PERI_CHAR_UUID)
                {
                    movPeriChar = characteristic
                }
                else
                {
                    print("SenseHub: Unknown characteristic UUID: \(characteristic.uuid.uuidString)")
                    continue
                }
            }
        }
        else
        {
            print("SenseHub: Unknown Service UUID \(service.uuid.uuidString)")
        }
        
        if(checkForCompleteChars() == true)
        {
            referencedController.timeoutTimer.invalidate()
            referencedController.connectionCon.dismiss(animated: true, completion: nil)
            setStateForAllSensors(state: true)
        }
    }
    
    /* Callback for a characteristic read/notification */
    func peripheral(_ peripheral: CBPeripheral,
                    didUpdateValueFor characteristic: CBCharacteristic,
                    error: Error?)
    {
        var values = [UInt8](repeating:0, count:characteristic.value!.count)
        var stats: SensorStatistics
        characteristic.value?.copyBytes(to: &values,
                                        count: characteristic.value!.count)
        
        if(sensorOverViewController == nil)
        {
            return
        }
        
        if(characteristic.uuid == TI_SN_IRTEMP_DATA_CHAR_UUID)
        {
            var floatValue: Float = 0.0
            memcpy(&floatValue, values, 4)
            stats = sensorOverViewController.addSensorData(sensor: "Temperature", data: floatValue)
            
            if(irTempGraphController != nil)
            {
                irTempGraphController.addGraphValue(floatValue: floatValue)
                irTempGraphController.updateStatistics(stats: stats)
            }
            
        }
        else if(characteristic.uuid == TI_SN_HUMIDITY_DATA_CHAR_UUID)
        {
            var floatValue: Float = 0.0
            memcpy(&floatValue, values, 4)
            stats = sensorOverViewController.addSensorData(sensor: "Humidity", data: floatValue)
            
            if(humidityGraphController != nil)
            {
                humidityGraphController.addGraphValue(floatValue: floatValue)
                humidityGraphController.updateStatistics(stats: stats)
            }
        }
        else if(characteristic.uuid == TI_SN_BAROMETER_DATA_CHAR_UUID)
        {
            var intValue: UInt32 = 0
            memcpy(&intValue, values, 4)
            stats = sensorOverViewController.addSensorData(sensor: "Pressure", data: Float(intValue)/1000.0)
            
            if(barometerGraphController != nil)
            {
                barometerGraphController.addGraphValue(floatValue: Float(intValue)/1000.0)
                barometerGraphController.updateStatistics(stats: stats)
            }
        }
        else if(characteristic.uuid == TI_SN_OPTIC_DATA_CHAR_UUID)
        {
            var floatValue: Float = 0.0
            memcpy(&floatValue, values, 4)
            stats = sensorOverViewController.addSensorData(sensor: "Optical", data: floatValue)
            
            if(opticalGraphController != nil)
            {
                opticalGraphController.addGraphValue(floatValue: floatValue)
                opticalGraphController.updateStatistics(stats: stats)
            }
        }
        else if(characteristic.uuid == TI_SN_MOVE_DATA_CHAR_UUID)
        {
            var aclAccVals = [Int16](repeatElement(0, count: 3))
            var gyrAccVals = [Int16](repeatElement(0, count: 3))
            var magAccVals = [Int16](repeatElement(0, count: 3))
            memcpy(&gyrAccVals, values, 6)
            memcpy(&aclAccVals, Array(values[6...11]), 6)
            memcpy(&magAccVals, Array(values[12...17]), 6)
            
            sensorOverViewController.addMovementSensorData(sensor: "Magnet", data: magAccVals)
            sensorOverViewController.addMovementSensorData(sensor: "Position", data: gyrAccVals)
            sensorOverViewController.addMovementSensorData(sensor: "Movement", data: aclAccVals)
            
            if(gyrGraphController != nil)
            {
                gyrGraphController.addGraphValue(intValX: gyrAccVals[0], intValY: gyrAccVals[1], intValZ: gyrAccVals[2])
            }
            
            if(magGraphController != nil)
            {
                magGraphController.addGraphValue(intValX: magAccVals[0], intValY: magAccVals[1], intValZ: magAccVals[2])
            }
            
            if(accGraphController != nil)
            {
                accGraphController.addGraphValue(intValX: aclAccVals[0], intValY: aclAccVals[1], intValZ: aclAccVals[2])
            }
        }
        else
        {
            print("SenseHub: Characteristic read (\(characteristic.uuid.uuidString))")
        }
        
    }


    // MARK: Functions
    func checkForCompleteChars() -> Bool
    {
        if(tempPeriChar != nil && tempConfChar != nil && tempDataChar != nil && humPeriChar != nil && humDataChar != nil
            && humConfChar != nil && baroPeriChar != nil && baroDataChar != nil && baroConfChar != nil &&
             optPeriChar != nil && optConfChar != nil && optDataChar != nil && movDataChar != nil && movPeriChar != nil
                && movConfChar != nil)
        {
            return true
        }
        else
        {
            return false
        }
    }
    
    func setStateForAllSensors(state: Bool)
    {
        var dataArray: [UInt8]
        
        if(state == true)
        {
            dataArray = [0x01]
        }
        else
        {
            dataArray = [0x00]
        }
        
        var writeArray = Data(dataArray)
        
        btDevice.setNotifyValue(state, for: tempDataChar)
        btDevice.setNotifyValue(state, for: humDataChar)
        btDevice.setNotifyValue(state, for: baroDataChar)
        btDevice.setNotifyValue(state, for: optDataChar)
        btDevice.setNotifyValue(state, for: movDataChar)
        btDevice.writeValue(writeArray, for: tempConfChar, type: CBCharacteristicWriteType.withResponse)
        btDevice.writeValue(writeArray, for: humConfChar, type: CBCharacteristicWriteType.withResponse)
        btDevice.writeValue(writeArray, for: baroConfChar, type: CBCharacteristicWriteType.withResponse)
        btDevice.writeValue(writeArray, for: optConfChar, type: CBCharacteristicWriteType.withResponse)
        btDevice.writeValue(writeArray, for: movConfChar, type: CBCharacteristicWriteType.withResponse)
        
        dataArray = [0x60, 0x00]
        writeArray = Data(dataArray)
        btDevice.writeValue(writeArray, for: tempPeriChar, type: CBCharacteristicWriteType.withResponse)
        btDevice.writeValue(writeArray, for: humPeriChar, type: CBCharacteristicWriteType.withResponse)
        btDevice.writeValue(writeArray, for: baroPeriChar, type: CBCharacteristicWriteType.withResponse)
        btDevice.writeValue(writeArray, for: optPeriChar, type: CBCharacteristicWriteType.withResponse)
        btDevice.writeValue(writeArray, for: movPeriChar, type: CBCharacteristicWriteType.withResponse)
        
    }
    
    // MARK: Navigation
    override func viewWillDisappear(_ animated: Bool)
    {
        referencedController.btManager.scanForPeripherals(withServices: nil, options: nil)
        referencedController.btManager.cancelPeripheralConnection(self.btDevice)
    }

}
