/*
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

import Foundation

enum SimpleLinkDeviceType: Int, CustomStringConvertible
{
    case cc2640r2 = 0
    case msp432cc26xx = 1
    case cc3220 = 2
    case msp432cc312x = 3
    case cc13x0 = 4
    
    static var count: Int { return SimpleLinkDeviceType.cc13x0.hashValue + 1}
    
    var description: String
    {
        switch self
        {
        case .cc2640r2: return "SimpleLink™ CC2640R2 SDK"
        case .msp432cc26xx   : return "SimpleLink™ SDK Bluetooth Plugin"
        case .cc3220 : return "SimpleLink™ CC3220 SDK"
        case .msp432cc312x : return "SimpleLink™ 3x20 WiFi SDKs"
        case .cc13x0 : return "SimpleLink™ 13x0 SDK"
        }
    }
}

class SimpleLinkDevice: NSObject
{
    // MARK: - Member Variables
    var deviceType: SimpleLinkDeviceType
    var marketingString: String
    var imageIdentifier: String
    var productURL: String
    var supportURL: String
    var userGuideURL: String
    var enabled: Bool
    
    override var description: String
    {
        return "\(self.deviceType.description)"
    }
    
    init(deviceType: SimpleLinkDeviceType, marketingString: String = "", enabled: Bool = false,
         imageIdentifier: String = "", productURL: String = "http://www.ti.com",
         supportURL: String = "http://www.ti.com", userGuideURL: String = "http://www.ti.com")
    {
        self.deviceType = deviceType
        self.marketingString = marketingString
        self.imageIdentifier = ""
        self.enabled = enabled
        self.imageIdentifier = imageIdentifier
        self.productURL = productURL
        self.supportURL = supportURL
        self.userGuideURL = userGuideURL
    }
    
}
