/*
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

import UIKit

class StatusLogViewController: UIViewController
{
    // MARK: - Properties
    @IBOutlet weak var logArea: UITextView!
    @IBOutlet weak var progressBar: UIProgressView!
    @IBOutlet weak var doneButton: UIButton!
    @IBOutlet weak var percentLabel: UILabel!

    // MARK: - Variables
    var referencedController: OADViewController?
    let dateFormatter = DateFormatter()
    
    // MARK: - Actions
    @IBAction func doneButtonPressed(_ sender: Any)
    {
        if(self.doneButton.titleLabel?.text == "Done")
        {
            self.dismiss(animated: true, completion: referencedController?.completionForUpdate)
        }
        else if(self.doneButton.titleLabel?.text == "Cancel")
        {
            referencedController?.cancelUpdate()
            self.printLogMessage(message: "Firmware update canceled!")
            self.dismiss(animated: true, completion: nil)
        }
    }
    
    
    // MARK: - Functions
    override func viewDidLoad()
    {
        super.viewDidLoad()

        dateFormatter.dateFormat = "HH:mm:ss"
        doneButton.layer.cornerRadius = 8.0
        doneButton.clipsToBounds = true
    }

    override func didReceiveMemoryWarning()
    {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    func printLogMessage(message: String)
    {
        DispatchQueue.main.async
        {
            self.logArea.text = self.logArea.text + self.dateFormatter.string(from: Date()) + ":    " + message + "\n"
            let range = NSRange(location: self.logArea.text.characters.count-1,length: 0)
            self.logArea.scrollRangeToVisible(range)
        }

    }
    
    func updateProgressBar(progress: Float)
    {
        DispatchQueue.main.async
        {
            let progressFloat = progress
            self.progressBar.setProgress(progressFloat, animated: true)
            self.percentLabel.text = String(Int(progressFloat*100)) + "%"
        }
    }

    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?)
    {
        // Get the new view controller using segue.destinationViewController.
        // Pass the selected object to the new view controller.
    }
    
    override func viewDidAppear(_ animated: Bool)
    {
        referencedController?.initiateFirmwareUpdate()
    }

}
