/*
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

import UIKit

class WiFiProvisionTableViewController: UITableViewController
{
    // MARK: Structure Definition
    struct ProvisionMethod
    {
        var title = String()
        var subTitle = String()
        var image = UIImage()
    }
    
    // MARK: Provision Method List
    let provisioning =
    [
            ProvisionMethod(title: "Direct Access Point",
                            subTitle: "Connect to device operating in AP mode by use direct use of local WiFi.",
                            image: UIImage(named: "router")!),
           /* ProvisionMethod(title: "BLE Provisioning",
                            subTitle: "Use a BLE connection via a CC26xx device to setup and provision your WiFi device.",
                            image: UIImage(named: "simpleap")!),*/
    ]
    
    // MARK: Variables
    let cellIdentifier = "ProvisioningListViewCell"

    override func viewDidLoad()
    {
        super.viewDidLoad()
    }

    override func didReceiveMemoryWarning()
    {
        super.didReceiveMemoryWarning()
    }

    // MARK: - Table view data source

    override func numberOfSections(in tableView: UITableView) -> Int
    {
        return 1
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int
    {
        return provisioning.count
    }

    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell
    {
        let cell : ProvisionMethodTableViewCell =
            tableView.dequeueReusableCell(withIdentifier: cellIdentifier)! as! ProvisionMethodTableViewCell
        
        let provisioningMethod = provisioning[indexPath.row]
        
        cell.subTitle.text = provisioningMethod.subTitle
        cell.mainTitle.text = provisioningMethod.title
        cell.imageIcon.image = provisioningMethod.image
        cell.subTitle.adjustsFontSizeToFitWidth = true
        
        return cell
    }
    
    // MARK: UITableViewDelegate
    override func tableView(_ tableView: UITableView,
                   didSelectRowAt indexPath: IndexPath)
    {
        let provisionMethod = provisioning[indexPath.row]
        
        if(provisionMethod.title == "Direct Access Point")
        {
            self.performSegue(withIdentifier: "DirectProvisionSegue", sender: self)
        }
        else if(provisionMethod.title == "BLE Provisioning")
        {
            self.performSegue(withIdentifier: "BLEProvisionSegue", sender: self)
        }

    }
}
